#!/bin/bash

WORK_DIR=$(dirname "$0")

#default
LINE_A=rs485
LINE_B=rs485
MODEM=wb_4g

help () {
    echo
    echo "To install the controller, set the parameters:"
    echo "--no_migrate          Only upgrade mosquitto template. With this parameter, the following arguments will be ignored!!!"
    echo "-m, --move            Only move controller to new server. With this parameter, the following arguments will be ignored!!!"
    echo "-t, --type            TYPE       [ dev | stage | online ]"
    echo "-i, --id              NODE_ID    Some controller id without spaces."
    echo "-u, --mqtt_username   USERNAME   Username on MQTT Server."
    echo "-p, --mqtt_password   PASSWORD   Password on MQTT Server."
    echo "--line_a              PROTOCOL   First line protocol [ rs485 ]."
    echo "--line_b              PROTOCOL   First line protocol [ rs485 | uart ]."
    echo
    echo "-z, --zigbee          Use to support ZigBee."
    echo "--modem               TYPE [ wb_4g | wb_4g ]   Use to 4G modem."
    echo "-o, --offline_install DIRECTORY     Use to install IQ packages from local directory."
    echo "-r, --root_password   PASSWORD      Set custom root password."
    echo "--beta                Use BETA Debian-packages."
    echo
}


while [ -n "$1" ]
do
case "$1" in
-t|--type)
    NODE_TYPE="$2"
    shift ;;

-i|--id)
    NODE_ID="$2"
    shift ;;

-u|--mqtt_username)
    LOGIN="$2"
    shift ;;

-p|--mqtt_password)
    PASSWORD=$2
    shift ;;

-o|--offline_install)
    DEB_PACKAGES_DIR=$2
    shift ;;

--line_a)
    LINE_A=$2
    shift ;;

--line_b)
    LINE_B=$2
    shift ;;

-w|--zwave)
    ZWAVE=true ;;

-b|--zigbee)
    ZIGBEE=true ;;

-m|--move)
    ONLY_MOVE=true ;;

--modem)
    MODEM=$2
    shift ;;

-r|--root_password)
    ROOT_PWD="$2"
    shift ;;

--beta)
    BETA=true ;;

*) help ;;
esac
shift
done

TIMESTAMP=$(date +"%s")

if [ "$ONLY_MOVE" = true ]; then
    ZWAVE=
    ZIGBEE=
    DEB_PACKAGES_DIR=

    if [ -z "$NODE_ID" ]; then
        NODE_ID=$(cat /etc/hostname)
    fi
fi

OLD_NODE_ID=$(cat /etc/hostname)

if [ -z "$NODE_TYPE" ] || \
    [ -z "$NODE_ID" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ]; then
    help
    exit 1
fi

if ping ${NODE_TYPE}.iqnergy.com -c 1
then
  echo "Сервер доступен! Продолжаем..."
else
  echo "Ошибка! Сервер недоступен!"
  exit 2
fi


#######################################
# Переход к архивам stretch
echo 'deb http://archive.debian.org/debian/ stretch main
deb http://archive.debian.org/debian-security/ stretch/updates main' | tee /etc/apt/sources.list.d/debian-upstream.list
echo 'deb http://archive.debian.org/debian/ stretch-backports main' | tee /etc/apt/sources.list.d/stretch-backports.list
#######################################
#######################################


#######################################
# Установка недостающих пакетов и удаление лишних
if [ -z "$ONLY_MOVE" ]; then
apt-get update

apt-get install -y \
    apt-listchanges \
    rsync \
    wb-mcu-fw-updater \
    resolvconf \
    socat \
    python3 \
    python3-pip \
    apt-transport-https || exit 3
    pip3 install paho-mqtt && pip3 install pyserial || exit 4
fi
#######################################
#######################################


#######################################
# Настройка сетевых адаптеров
if [ -z "$ONLY_MOVE" ]; then
    $WORK_DIR/configure_hardware.py -z wb_cc2530 -m ${MODEM} -a ${LINE_A} -b ${LINE_B} || exit 5
fi
#######################################
#######################################

# Установка ПО IQnergy
if [ -z "$ONLY_MOVE" ]; then

    if [ -z "$BETA" ]; then
        echo 'deb https://deb.iqnergy.com/debian/ stretch modern_6_9' > /etc/apt/sources.list.d/iqnergy.list
    else
        echo 'deb https://deb.iqnergy.com/debian/ stretch beta' > /etc/apt/sources.list.d/iqnergy.list
    fi

    wget -O - http://deb.iqnergy.com/signature.gpg.key | apt-key add - || exit 8
    systemctl restart apt-daily.timer || exit 9
    apt-get -y update

    if [ -z "$DEB_PACKAGES_DIR" ]; then
        export DEBIAN_FRONTEND=noninteractive ; apt-get -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" install -y iq-iqnergy-software-installer || exit 10
    else
        export DEBIAN_FRONTEND=noninteractive ; apt-get -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" install -y "$DEB_PACKAGES_DIR"*deb -y || exit 11
    fi

    # конфигурируем mosquitto
    "$WORK_DIR"/configure_mosquitto.sh --mqtt_username ${LOGIN} --mqtt_password ${PASSWORD} --type ${NODE_TYPE} --id ${NODE_ID} --port 5050 || exit 6

    systemctl restart mosquitto || exit 7

fi
#######################################
#######################################


#######################################
# Установка ZigBee
if [ "$ZIGBEE" = true ]; then
    ZIGBEE_DIR=/mnt/data/root/zigbee2mqtt
    apt install -y zigbee2mqtt wb-zigbee2mqtt

    CH=`expr ${NODE_ID:2} % 4`

    case $CH in

      0)
        ZIGBEE_CHANNEL=11
        ;;

      1)
        ZIGBEE_CHANNEL=15
        ;;

      2)
        ZIGBEE_CHANNEL=20
        ;;

      *)
        ZIGBEE_CHANNEL=25
        ;;
    esac


    ZIGBEE_PANID=`expr ${NODE_ID:2} % 16383`
    DEFAULT_ZIGBEE_KEY='DEFAULT!ZigbeeKey'
    ANSII_KEY=`echo $NODE_ID$NODE_ID$DEFAULT_ZIGBEE_KEY | cut -c1-16`
    ANSII_EXT_PANID=`echo $NODE_ID$NODE_ID$DEFAULT_ZIGBEE_KEY | cut -c1-8`
    ZIGBEE_NETWORK_KEY=`printf "$ANSII_KEY" | xxd -p | sed 's/\(..\)/0x&, /g; s/, $//;'`
    ZIGBEE_EXT_PANID=`printf "$ANSII_EXT_PANID" | xxd -p | sed 's/\(..\)/0x&, /g; s/, $//;'`

    echo -e "$(eval "echo -e \"`cat \"$WORK_DIR/zigbee/configuration.yaml.template\"`\"")" | tee "$ZIGBEE_DIR/data/configuration.yaml" || exit 12
    cp "$WORK_DIR/zigbee/iqnergy.js" "$ZIGBEE_DIR/node_modules/zigbee-herdsman-converters/devices/"
fi
#######################################
#######################################


#######################################
# Меняем пароль по-умолчанию на свой
if [ -z "$ONLY_MOVE" ]; then
    echo -e "\nCHANGING ROOT PASSWORD AND ADDING \"$USER_NAME\" USER"
    echo "New password for ROOT"
    echo -e "$ROOT_PWD\n$ROOT_PWD" | passwd || exit 13
fi
#######################################
#######################################


######################################
# Запоминаем время
if [ -z "$ONLY_MOVE" ]; then
    hwclock --systohc --localtime
fi
######################################
######################################

apt-get clean

cat "$WORK_DIR/IQ_logo.txt"
sync

YELLOW='\033[1;33m'
GREEN='\033[1;32m'
NC='\033[0m'

printf "Old ID: ${YELLOW}$OLD_NODE_ID${NC}\n"
printf "New ID: ${YELLOW}$(cat /etc/hostname)${NC}\nServer: ${YELLOW}$NODE_TYPE${NC}\n"
printf "Revision: ${YELLOW}$(timeout 5 mosquitto_sub -t "/devices/system/controls/Batch No" -C 1)${NC}\n"
printf "SN: ${YELLOW}$(timeout 5 mosquitto_sub -t "/devices/system/controls/Short SN" -C 1)${NC}\n"
printf "${GREEN}Install done! ${NC}\n"
echo Reboot...

sync; sync; sleep 1
reboot
