#!/bin/bash
# Скрипт получает все retained-сообщения из топика /iq/bridge/rules/add/+
# и для каждого извлекает ID (часть после /iq/bridge/rules/add/),
# затем проверяет наличие файла в каталоге /etc/wb-rules/,
# имя которого содержит этот ID.
# Если хотя бы для одного ID соответствующий файл не найден, скрипт завершится с ошибкой.

missing=0

# Используем timeout, чтобы mosquitto_sub работал 10 секунд и завершался,
# когда сообщения закончатся.
# Режим -v выводит топик и сообщение, поэтому с помощью awk извлекаем только первое слово (топик).
while IFS= read -r line; do
    # Извлекаем топик (первое слово строки)
    topic=$(echo "$line" | awk '{print $1}')

    # Если строка пуста — пропускаем
    [ -z "$topic" ] && continue

    # Удаляем префикс "/iq/bridge/rules/add/" чтобы получить ID
    id="${topic#/iq/bridge/rules/add/}"

    # Если ID не получен, пропускаем строку
    [ -z "$id" ] && continue

    # Проверяем, существует ли хотя бы один файл в /etc/wb-rules/,
    # имя которого содержит данный ID.
    if ls /etc/wb-rules/*"$id"* >/dev/null 2>&1; then
        echo "Файл найден для ID: $id"
    else
        echo "Файл не найден для ID: $id"
        missing=1
    fi

done < <(timeout 10 mosquitto_sub -t '/iq/bridge/rules/add/+' -v | grep '/iq/bridge/rules/add/')

# Если хотя бы один файл не найден, возвращаем ошибку
if [ $missing -eq 1 ]; then
    exit 1
else
    exit 0
fi
